﻿// Copyright (c) 2013, Simon Carter.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted 
// provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright notice, this list of 
//   conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright notice, this list of 
//  conditions and the following disclaimer in the documentation and/or other materials provided 
//  with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// History
// Initial Release - Simon Carter
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;

using TCPMessageServer;

namespace ErrorManager.Console.Controls
{
    /// <summary>
    /// Base control class for all internal controls
    /// </summary>
    public class BaseControlClass : UserControl
    {
        #region Private Members

        private MessageClient _client;

        #endregion Private Members

        #region Internal Methods

        /// <summary>
        /// Initialises the control
        /// </summary>
        /// <param name="client">Active MessageClient object</param>
        internal virtual void Initialise(MessageClient client)
        {
            if (client == null)
                throw new ArgumentNullException("MessageClient can not be null");

            _client = client;
            _client.MessageReceived += _client_MessageReceived;
        }

        #endregion Internal Methods

        #region Internal Properties

        /// <summary>
        /// Returns the active MessageClient object
        /// </summary>
        internal MessageClient Client
        {
            get
            {
                return (_client);
            }
        }

        /// <summary>
        /// Returns the name of the tab on the console
        /// </summary>
        internal virtual string TabName
        {
            get
            {
                throw new NotOverriddenException();
            }
        }

        /// <summary>
        /// Name of Permission required to access the control
        /// </summary>
        internal virtual string Permission
        {
            get
            {
                throw new NotOverriddenException();
            }
        }

        #endregion Internal Properties

        #region Abstract Methods

        /// <summary>
        /// Virtual method to receive messages
        /// 
        /// Controls which need to receive messages need to override this method
        /// </summary>
        /// <param name="message">Message being received</param>
        internal virtual void ProcessMessage(TCPMessageServer.Message message)
        {
        }

        #endregion Abstract Methods

        #region Private Methods

        /// <summary>
        /// Receives messages from the MessageClient
        /// </summary>
        /// <param name="sender">sender</param>
        /// <param name="message">Message being received</param>
        private void _client_MessageReceived(object sender, TCPMessageServer.Message message)
        {
            ProcessMessage(message);
        }

        #endregion Private Methods

        #region Public Methods

        public string GetXMLValue(string root, string searchTerm, string defaultValue = "")
        {
            string Result = String.Empty;

            //Load xml
            XDocument xdoc = XDocument.Load("ErrorServerConfig.xml");

            //Run query
            var lv1s = from lv1 in xdoc.Descendants(root)
                       select new
                       {
                           Header = lv1.Elements(searchTerm).ToString(),
                           Children = lv1.Descendants(searchTerm)
                       };

            //Loop through results
            foreach (var lv1 in lv1s)
            {
                foreach (var lv2 in lv1.Children)
                    Result = lv2.Value;
            }

            return (String.IsNullOrEmpty(Result) ? defaultValue : Result);
        }

        #endregion Public Methods
    }
}
